from datetime import datetime, timedelta

from cg.utils.date import (
    convert_string_to_datetime_object,
    get_date_days_ago,
    get_timedelta_from_date,
)


def test_get_date_days_ago(timestamp_now: datetime):
    """Test returning a date corresponding to number of days ago."""

    # GIVEN days ago

    # WHEN calling the function
    date: datetime = get_date_days_ago(days_ago=1)

    # THEN assert the return should be a date
    assert isinstance(date, datetime)

    # Then the date returned should be less than then today
    assert date < timestamp_now


def test_get_timedelta_from_date(timestamp_yesterday: datetime):
    """Test returning the number of days ago corresponding to a given date."""

    # GIVEN a date

    # WHEN calling the function
    age: timedelta = get_timedelta_from_date(date=timestamp_yesterday)

    # THEN assert the return should be a time delta
    assert isinstance(age, timedelta)

    # Then the age in days should be yesterday
    assert age.days == 1


def test_convert_string_to_datetime_object_valid():
    # GIVEN a list of valid datetime strings with different formats
    valid_datetime_strings = [
        ("10/15/2024 12:45:30", "%m/%d/%Y %H:%M:%S"),  # Month/Day/Year, U.S. format
    ]

    # WHEN the function is called with valid strings
    for date_str, expected_format in valid_datetime_strings:
        # THEN it should return the correct datetime object
        expected_datetime = datetime.strptime(date_str, expected_format)
        assert convert_string_to_datetime_object(date_str) == expected_datetime


def test_convert_string_to_datetime_object_invalid():
    # GIVEN a list of invalid datetime strings
    invalid_datetime_strings = [
        "2024-15-10 12:45:30",  # Invalid day format
        "15-10-2024 12:45",  # Missing seconds
        "Invalid string",  # Completely invalid
    ]

    # WHEN the function is called with invalid strings
    for date_str in invalid_datetime_strings:
        # THEN it should raise a ValueError
        try:
            convert_string_to_datetime_object(date_str)
            assert False, f"Expected ValueError for {date_str} but it didn't raise"
        except ValueError as e:
            assert str(e) == f"Could not convert '{date_str}' to a datetime object."



"""Module to test the arnold api client."""

import pytest
import requests
from pytest_mock import MockFixture

from cg.clients.arnold.api import ArnoldAPIClient
from cg.clients.arnold.dto.create_case_request import CreateCaseRequest
from cg.clients.arnold.exceptions import ArnoldClientError


def test_create_case_successful(
    mocker: MockFixture,
    arnold_client: ArnoldAPIClient,
    create_case_request: CreateCaseRequest,
    mock_post_request_ok: MockFixture,
):
    # GIVEN a mocked response from the requests.post method that is successful
    mocked_post = mocker.patch.object(requests, "post")
    mocked_post.return_value = mock_post_request_ok

    # WHEN creating a case
    arnold_client.create_case(create_case_request)

    # THEN  a case request is sent
    mocked_post.assert_called_once_with(
        f"{arnold_client.api_url}/case/", data=create_case_request.model_dump_json(), verify=True
    )


def test_create_case_not_successful(
    mocker: MockFixture,
    arnold_client: ArnoldAPIClient,
    create_case_request: CreateCaseRequest,
    mock_post_request_not_found: MockFixture,
    error_content: str,
):
    # GIVEN a mocked response from the requests.post method that is not successful
    mocked_post = mocker.patch.object(requests, "post")
    mocked_post.return_value = mock_post_request_not_found

    # WHEN creating a case

    # THEN the request raises an error
    with pytest.raises(ArnoldClientError):
        arnold_client.create_case(create_case_request)



import logging
from pathlib import Path

from cg.utils.checksum.checksum import is_md5sum_correct

LOG = logging.getLogger(__name__)


def are_all_fastq_valid(fastq_paths: list[Path]) -> bool:
    """Return True if all fastq files of a given sample pass md5 checksum."""
    are_all_valid: bool = True
    for path in fastq_paths:
        if not is_md5sum_correct(path):
            are_all_valid = False
            LOG.warning(f"Sample {path} did not match the given md5sum")
    return are_all_valid



"""Module to test the PacBioStoreService."""

from unittest import mock

import pytest

from cg.services.run_devices.exc import (
    PostProcessingDataTransferError,
    PostProcessingStoreDataError,
)
from cg.services.run_devices.pacbio.data_storage_service.pacbio_store_service import (
    PacBioStoreService,
)
from cg.services.run_devices.pacbio.data_transfer_service.data_transfer_service import (
    PacBioDataTransferService,
)
from cg.services.run_devices.pacbio.data_transfer_service.dto import PacBioDTOs
from cg.services.run_devices.pacbio.run_data_generator.run_data import PacBioRunData
from cg.store.models import (
    PacbioSampleSequencingMetrics,
    PacbioSequencingRun,
    PacbioSMRTCell,
    Sample,
)
from cg.store.store import Store
from tests.store_helpers import StoreHelpers


def test_store_post_processing_data(
    pac_bio_store_service: PacBioStoreService,
    pac_bio_dtos: PacBioDTOs,
    pacbio_barcoded_run_data: PacBioRunData,
    helpers: StoreHelpers,
):
    # GIVEN a PacBioStoreService and a data transfer service

    # GIVEN that the store has a sample
    helpers.add_sample(
        store=pac_bio_store_service.store,
        internal_id=pac_bio_dtos.sample_sequencing_metrics[0].sample_internal_id,
    )

    # GIVEN that the store has no sequencing data
    assert not pac_bio_store_service.store._get_query(PacbioSampleSequencingMetrics).first()
    assert not pac_bio_store_service.store._get_query(PacbioSequencingRun).first()
    assert not pac_bio_store_service.store._get_query(PacbioSMRTCell).first()

    # GIVEN a data transfer service that returns the correct DTOs

    # WHEN storing data for a PacBio instrument run
    with mock.patch(
        "cg.services.run_devices.pacbio.data_transfer_service.data_transfer_service.PacBioDataTransferService.get_post_processing_dtos",
        return_value=pac_bio_dtos,
    ):
        pac_bio_store_service.store_post_processing_data(pacbio_barcoded_run_data)

    # THEN the SMRT cell data is stored with the correct data
    smrt_cell: PacbioSMRTCell = pac_bio_store_service.store._get_query(PacbioSMRTCell).first()
    assert smrt_cell
    assert smrt_cell.internal_id == pac_bio_dtos.run_device.internal_id

    # THEN the sequencing run is stored with the correct data
    sequencing_run: PacbioSequencingRun = pac_bio_store_service.store._get_query(
        PacbioSequencingRun
    ).first()
    assert sequencing_run
    assert sequencing_run.well == pac_bio_dtos.sequencing_run.well

    # THEN the sample sequencing metrics are stored with the correct data
    sample_sequencing_run_metrics: list[PacbioSampleSequencingMetrics] = (
        pac_bio_store_service.store._get_query(PacbioSampleSequencingMetrics).all()
    )
    assert sample_sequencing_run_metrics
    for sample_metrics in sample_sequencing_run_metrics:
        assert (
            sample_metrics.sample.internal_id
            == pac_bio_dtos.sample_sequencing_metrics[0].sample_internal_id
        )

    # THEN the sample reads and sequenced date are updated
    for sample_metrics_dto in pac_bio_dtos.sample_sequencing_metrics:
        sample: Sample = pac_bio_store_service.store.get_sample_by_internal_id(
            sample_metrics_dto.sample_internal_id
        )
        assert sample
        assert sample.reads == sample_metrics_dto.hifi_reads
        assert sample.last_sequenced_at == pac_bio_dtos.sequencing_run.completed_at


def test_store_post_processing_data_error_database(
    pac_bio_store_service: PacBioStoreService,
    pac_bio_dtos: PacBioDTOs,
    pacbio_barcoded_run_data: PacBioRunData,
):
    # GIVEN a PacbioStoreService and a Pacbio run data object

    # GIVEN a store that raises an error when creating a PacBio SMRT cell

    # WHEN trying to store data for a Pacbio instrument run
    with mock.patch(
        "cg.services.run_devices.pacbio.data_transfer_service.data_transfer_service.PacBioDataTransferService.get_post_processing_dtos",
        return_value=pac_bio_dtos,
    ), mock.patch.object(Store, "create_pac_bio_smrt_cell", side_effect=ValueError):
        # THEN a PostProcessingStoreDataError is raised
        with pytest.raises(PostProcessingStoreDataError):
            pac_bio_store_service.store_post_processing_data(pacbio_barcoded_run_data)


def test_store_post_processing_data_error_parser(
    pac_bio_store_service: PacBioStoreService,
    pac_bio_dtos: PacBioDTOs,
    pacbio_barcoded_run_data: PacBioRunData,
):
    # GIVEN a PacbioStoreService and a Pacbio run data object

    # GIVEN a data transfer service that raises an error when parsing data

    # WHEN trying to store data for a PacBio instrument ru
    with mock.patch(
        "cg.services.run_devices.pacbio.data_transfer_service.data_transfer_service.PacBioDataTransferService.get_post_processing_dtos",
        return_value=pac_bio_dtos,
    ), mock.patch.object(
        PacBioDataTransferService,
        "get_post_processing_dtos",
        side_effect=PostProcessingDataTransferError,
    ):
        # THEN a PostProcessingStoreDataError is raised
        with pytest.raises(PostProcessingStoreDataError):
            pac_bio_store_service.store_post_processing_data(pacbio_barcoded_run_data)



import logging
from datetime import datetime

from cg.constants import Priority, Workflow
from cg.constants.constants import CaseActions, DataDelivery
from cg.constants.pedigree import Pedigree
from cg.models.orders.order import OrderIn
from cg.models.orders.samples import Of1508Sample
from cg.services.orders.order_lims_service.order_lims_service import OrderLimsService
from cg.services.orders.submitters.order_submitter import StoreOrderService
from cg.store.models import (
    ApplicationVersion,
    Case,
    CaseSample,
    Customer,
    Order,
    Sample,
)
from cg.store.store import Store

LOG = logging.getLogger(__name__)


class StoreCaseOrderService(StoreOrderService):
    """
    Service for storing generic orders in StatusDB and Lims.
    This class is used to store orders for the following workflows:
    - Balsamic
    - Balsamic QC
    - Balsamic UMI
    - MIP DNA
    - MIP RNA
    - Tomte
    """

    def __init__(
        self,
        status_db: Store,
        lims_service: OrderLimsService,
    ):
        self.status_db = status_db
        self.lims = lims_service

    def store_order(self, order: OrderIn) -> dict:
        """Submit a batch of samples for sequencing and analysis."""
        return self._process_case_samples(order=order)

    def _process_case_samples(self, order: OrderIn) -> dict:
        """Process samples to be analyzed."""
        project_data = lims_map = None

        # submit new samples to lims
        new_samples = [sample for sample in order.samples if sample.internal_id is None]
        if new_samples:
            project_data, lims_map = self.lims.process_lims(
                lims_order=order, new_samples=new_samples
            )

        status_data = self.order_to_status(order=order)
        samples = [sample for family in status_data["families"] for sample in family["samples"]]
        if lims_map:
            self._fill_in_sample_ids(samples=samples, lims_map=lims_map)

        new_cases: list[Case] = self.store_items_in_status(
            customer_id=status_data["customer"],
            order=status_data["order"],
            ordered=project_data["date"] if project_data else datetime.now(),
            ticket_id=order.ticket,
            items=status_data["families"],
        )
        return {"project": project_data, "records": new_cases}

    @staticmethod
    def _group_cases(samples: list[Of1508Sample]) -> dict:
        """Group samples in cases."""
        cases = {}
        for sample in samples:
            case_id = sample.family_name
            if case_id not in cases:
                cases[case_id] = []
            cases[case_id].append(sample)
        return cases

    @staticmethod
    def _get_single_value(case_name, case_samples, value_key, value_default=None):
        values = set(getattr(sample, value_key) or value_default for sample in case_samples)
        if len(values) > 1:
            raise ValueError(f"different sample {value_key} values: {case_name} - {values}")
        single_value = values.pop()
        return single_value

    def order_to_status(self, order: OrderIn) -> dict:
        """Converts order input to status interface input for MIP-DNA, MIP-RNA and Balsamic."""
        status_data = {"customer": order.customer, "order": order.name, "families": []}
        cases = self._group_cases(order.samples)

        for case_name, case_samples in cases.items():
            case_internal_id: str = self._get_single_value(
                case_name, case_samples, "case_internal_id"
            )
            cohorts: set[str] = {
                cohort for sample in case_samples for cohort in sample.cohorts if cohort
            }
            data_analysis = self._get_single_value(case_name, case_samples, "data_analysis")
            data_delivery = self._get_single_value(case_name, case_samples, "data_delivery")

            panels: set[str] = set()
            if data_analysis in [Workflow.MIP_DNA, Workflow.TOMTE]:
                panels: set[str] = {
                    panel for sample in case_samples for panel in sample.panels if panel
                }

            priority = self._get_single_value(
                case_name, case_samples, "priority", Priority.standard.name
            )
            synopsis: str = self._get_single_value(case_name, case_samples, "synopsis")

            case = {
                "cohorts": list(cohorts),
                "data_analysis": data_analysis,
                "data_delivery": data_delivery,
                "internal_id": case_internal_id,
                "name": case_name,
                "panels": list(panels),
                "priority": priority,
                "samples": [
                    {
                        "age_at_sampling": sample.age_at_sampling,
                        "application": sample.application,
                        "capture_kit": sample.capture_kit,
                        "comment": sample.comment,
                        "control": sample.control,
                        "father": sample.father,
                        "internal_id": sample.internal_id,
                        "mother": sample.mother,
                        "name": sample.name,
                        "phenotype_groups": list(sample.phenotype_groups),
                        "phenotype_terms": list(sample.phenotype_terms),
                        "reference_genome": (
                            sample.reference_genome if hasattr(sample, "reference_genome") else None
                        ),
                        "sex": sample.sex,
                        "status": sample.status if hasattr(sample, "status") else None,
                        "subject_id": sample.subject_id,
                        "tumour": sample.tumour,
                    }
                    for sample in case_samples
                ],
                "synopsis": synopsis,
            }

            status_data["families"].append(case)
        return status_data

    def store_items_in_status(
        self, customer_id: str, order: str, ordered: datetime, ticket_id: str, items: list[dict]
    ) -> list[Case]:
        """Store cases, samples and their relationship in the Status database."""
        customer: Customer = self.status_db.get_customer_by_internal_id(
            customer_internal_id=customer_id
        )
        new_cases: list[Case] = []
        status_db_order = Order(
            customer=customer,
            order_date=datetime.now(),
            ticket_id=int(ticket_id),
        )
        for case in items:
            status_db_case: Case = self.status_db.get_case_by_internal_id(
                internal_id=case["internal_id"]
            )
            if not status_db_case:
                new_case: Case = self._create_case(
                    case=case, customer_obj=customer, ticket=ticket_id
                )
                new_cases.append(new_case)
                self._update_case_panel(panels=case["panels"], case=new_case)
                status_db_case: Case = new_case
            else:
                self._append_ticket(ticket_id=ticket_id, case=status_db_case)
                self._update_action(action=CaseActions.ANALYZE, case=status_db_case)
                self._update_case_panel(panels=case["panels"], case=status_db_case)
            case_samples: dict[str, Sample] = {}
            status_db_order.cases.append(status_db_case)
            for sample in case["samples"]:
                existing_sample: Sample = self.status_db.get_sample_by_internal_id(
                    internal_id=sample["internal_id"]
                )
                if not existing_sample:
                    new_sample: Sample = self._create_sample(
                        case=case,
                        customer_obj=customer,
                        order=order,
                        ordered=ordered,
                        sample=sample,
                        ticket=ticket_id,
                    )
                    case_samples[sample["name"]] = new_sample
                else:
                    case_samples[sample["name"]] = existing_sample

            for sample in case["samples"]:
                sample_mother: Sample = case_samples.get(sample.get(Pedigree.MOTHER))
                sample_father: Sample = case_samples.get(sample.get(Pedigree.FATHER))
                with self.status_db.session.no_autoflush:
                    case_sample: CaseSample = self.status_db.get_case_sample_link(
                        case_internal_id=status_db_case.internal_id,
                        sample_internal_id=sample["internal_id"],
                    )
                if not case_sample:
                    case_sample: CaseSample = self._create_link(
                        case_obj=status_db_case,
                        family_samples=case_samples,
                        father_obj=sample_father,
                        mother_obj=sample_mother,
                        sample=sample,
                    )

                self._update_relationship(
                    father_obj=sample_father,
                    link_obj=case_sample,
                    mother_obj=sample_mother,
                    sample=sample,
                )
            self.status_db.session.add_all(new_cases)
            self.status_db.session.add(status_db_order)
            self.status_db.session.commit()
        return new_cases

    @staticmethod
    def _update_case_panel(panels: list[str], case: Case) -> None:
        """Update case panels."""
        case.panels = panels

    @staticmethod
    def _append_ticket(ticket_id: str, case: Case) -> None:
        """Add a ticket to the case."""
        case.tickets = f"{case.tickets},{ticket_id}"

    @staticmethod
    def _update_action(action: str, case: Case) -> None:
        """Update action of a case."""
        case.action = action

    @staticmethod
    def _update_relationship(father_obj, link_obj, mother_obj, sample):
        link_obj.status = sample["status"] or link_obj.status
        link_obj.mother = mother_obj or link_obj.mother
        link_obj.father = father_obj or link_obj.father

    def _create_link(self, case_obj, family_samples, father_obj, mother_obj, sample):
        link_obj = self.status_db.relate_sample(
            case=case_obj,
            sample=family_samples[sample["name"]],
            status=sample["status"],
            mother=mother_obj,
            father=father_obj,
        )
        self.status_db.session.add(link_obj)
        return link_obj

    def _create_sample(self, case, customer_obj, order, ordered, sample, ticket):
        sample_obj = self.status_db.add_sample(
            name=sample["name"],
            comment=sample["comment"],
            control=sample["control"],
            internal_id=sample["internal_id"],
            order=order,
            ordered=ordered,
            original_ticket=ticket,
            tumour=sample["tumour"],
            age_at_sampling=sample["age_at_sampling"],
            capture_kit=sample["capture_kit"],
            phenotype_groups=sample["phenotype_groups"],
            phenotype_terms=sample["phenotype_terms"],
            priority=case["priority"],
            reference_genome=sample["reference_genome"],
            sex=sample["sex"],
            subject_id=sample["subject_id"],
        )
        sample_obj.customer = customer_obj
        with self.status_db.session.no_autoflush:
            application_tag = sample["application"]
            sample_obj.application_version: ApplicationVersion = (
                self.status_db.get_current_application_version_by_tag(tag=application_tag)
            )
        self.status_db.session.add(sample_obj)
        return sample_obj

    def _create_case(self, case: dict, customer_obj: Customer, ticket: str):
        case_obj = self.status_db.add_case(
            cohorts=case["cohorts"],
            data_analysis=Workflow(case["data_analysis"]),
            data_delivery=DataDelivery(case["data_delivery"]),
            name=case["name"],
            priority=case["priority"],
            synopsis=case["synopsis"],
            ticket=ticket,
        )
        case_obj.customer = customer_obj
        return case_obj

    @staticmethod
    def _is_rerun_of_existing_case(sample: Of1508Sample) -> bool:
        return sample.case_internal_id is not None



from click import Context

from cg.meta.upload.fohm.fohm import FOHMUploadAPI
from cg.meta.upload.gisaid import GisaidAPI
from cg.meta.upload.upload_api import UploadAPI
from cg.meta.workflow.mutant import MutantAnalysisAPI
from cg.models.cg_config import CGConfig
from cg.store.models import Analysis, Case


class MutantUploadAPI(UploadAPI):

    def __init__(self, config: CGConfig):
        self.analysis_api: MutantAnalysisAPI = MutantAnalysisAPI(config)
        self.fohm_api = FOHMUploadAPI(config)
        self.gsaid_api = GisaidAPI(config)

        super().__init__(config=config, analysis_api=self.analysis_api)

    def upload(self, ctx: Context, case: Case, restart: bool) -> None:
        latest_analysis: Analysis = case.analyses[0]
        self.update_upload_started_at(latest_analysis)
        self.upload_files_to_customer_inbox(case)
        self.gsaid_api.upload(case.internal_id)
        self.fohm_api.aggregate_delivery(case_ids=[case.internal_id])
        self.fohm_api.sync_files_sftp()
        self.update_uploaded_at(latest_analysis)



import math
from pathlib import Path

import pytest
from _pytest.fixtures import FixtureRequest

from cg.constants.demultiplexing import DemultiplexingDirsAndFiles
from cg.constants.sequencing import FLOWCELL_Q30_THRESHOLD, Sequencers
from cg.services.illumina.data_transfer.models import (
    IlluminaSampleSequencingMetricsDTO,
)
from cg.services.illumina.post_processing.utils import (
    _add_flow_cell_name_to_fastq_file_path,
    _combine_metrics,
    _get_valid_sample_fastqs,
    _get_weighted_average,
    _is_file_path_compressed_fastq,
    _is_lane_in_fastq_file_name,
    _is_sample_id_in_directory_name,
    _is_valid_sample_fastq_file,
    combine_sample_metrics_with_undetermined,
    create_delivery_file_in_flow_cell_directory,
    get_lane_from_sample_fastq,
    get_q30_threshold,
    get_sample_fastqs_from_flow_cell,
    get_undetermined_fastqs,
    is_sample_negative_control_with_reads_in_lane,
)


@pytest.mark.parametrize(
    "reads, is_negative_control, expected_negative_control_with_reads",
    [
        (0, False, False),
        (0, True, False),
        (1, True, True),
        (1, False, False),
    ],
    ids=[
        "no_reads_no_negative_control",
        "no_reads_negative_control",
        "reads_negative_control",
        "reads_no_negative_control",
    ],
)
def test_is_sample_negative_control_with_reads_in_lane(
    mapped_metric: IlluminaSampleSequencingMetricsDTO,
    reads: int,
    is_negative_control: bool,
    expected_negative_control_with_reads: bool,
):
    # GIVEN a metric with a specific amount of reads for a sample
    mapped_metric.total_reads_in_lane = reads

    # WHEN checking if the sample is a negative control with reads in the lane
    result: bool = is_sample_negative_control_with_reads_in_lane(
        is_negative_control=is_negative_control, metric=mapped_metric
    )

    # THEN the result should be return according to the expected outcome
    assert result == expected_negative_control_with_reads


@pytest.mark.parametrize(
    "directory, expected_result",
    [
        (Path("Sample_123"), True),
        (Path("sample/123_L0002.fastq.gz"), False),
    ],
    ids=["valid_directory", "invalid_directory"],
)
def test_is_sample_id_in_directory(directory: Path, expected_result: bool):
    # GIVEN a directory and a sample internal id
    sample_internal_id: str = "123"

    # WHEN checking if the sample id is in the directory name
    is_sample_id_in_dir: bool = _is_sample_id_in_directory_name(
        directory=directory, sample_internal_id=sample_internal_id
    )

    # THEN the result should be the expected
    assert is_sample_id_in_dir == expected_result


@pytest.mark.parametrize(
    "file_path, expected_result",
    [
        (Path("sample_L0002.fastq.gz"), True),
        (Path("sample_L0002.fastq"), False),
    ],
    ids=["compressed_file", "uncompressed_file"],
)
def test_is_file_path_compressed_fastq_with_valid_file(file_path: Path, expected_result: bool):
    # GIVEN a file

    # WHEN checking if the file path is a compressed fastq file
    is_file_compressed: bool = _is_file_path_compressed_fastq(file_path)

    # THEN the result is the expected
    assert is_file_compressed == expected_result


@pytest.mark.parametrize(
    "file_path, expected_result",
    [
        (Path("sample_L0002.fastq.gz"), True),
        (Path("sample.fastq.gz"), False),
    ],
    ids=["valid_file", "invalid_file"],
)
def test_is_lane_in_fastq_file_name_with_valid_file(file_path: Path, expected_result: bool):
    # GIVEN a valid file containing lane number

    # WHEN checking if the lane number is in the fastq file name
    is_lane_in_name: bool = _is_lane_in_fastq_file_name(file_path)

    # THEN the result should be the expected
    assert is_lane_in_name == expected_result


@pytest.mark.parametrize(
    "sample_fastq, expected_result",
    [
        (Path("Sample_123/sample_L0002.fastq.gz"), True),
        (Path("L0002.fastq.gz"), False),
        (Path("Sample_123/sample.fastq.gz"), False),
        (Path("Sample_123/123_L0002.fastq"), False),
    ],
    ids=["valid_file", "no_sample_id", "no_lane", "uncompressed"],
)
def test_is_valid_sample_fastq_file(sample_fastq: Path, expected_result: bool):
    # GIVEN a fastq file and a sample internal id
    sample_internal_id = "sample"

    # WHEN checking if the sample fastq file is valid
    is_valid_fastq: bool = _is_valid_sample_fastq_file(
        sample_fastq=sample_fastq, sample_internal_id=sample_internal_id
    )

    # THEN the result should be the expected
    assert is_valid_fastq == expected_result


def test_get_valid_sample_fastqs():
    # GIVEN a list of fastq paths with one valid and one invalid path
    fastq_paths: list[Path] = [Path("Sample_123/sample_L0002.fastq.gz"), Path("L0002.fastq.gz")]

    # WHEN getting the valid fastq files
    valid_fastqs: list[Path] = _get_valid_sample_fastqs(
        fastq_paths=fastq_paths, sample_internal_id="123"
    )

    # THEN the result should be the valid fastq file
    assert len(valid_fastqs) == 1
    assert valid_fastqs[0] == Path("Sample_123/sample_L0002.fastq.gz")


@pytest.mark.parametrize(
    "demux_run_path_fixture, sample_internal_id",
    [
        ("tmp_demultiplexed_novaseq_6000_post_1_5_kits_path", "ACC12642A7"),
        ("novaseq_x_demux_runs_dir", "ACC13169A1"),
    ],
    ids=["demuxed_on_dragen", "demuxed_on_sequencer"],
)
def test_get_sample_fastqs_from_flow_cell(
    demux_run_path_fixture: str, sample_internal_id: str, request: FixtureRequest
):
    # GIVEN a demultiplexed run path and a sample internal id
    demux_run_path: Path = request.getfixturevalue(demux_run_path_fixture)

    # WHEN getting the sample fastq files from the flow cell directory
    sample_fastqs: list[Path] = get_sample_fastqs_from_flow_cell(
        demultiplexed_run_path=demux_run_path, sample_internal_id=sample_internal_id
    )

    # THEN valid sample fastq files should be returned
    assert len(sample_fastqs) == 2
    for fastq in sample_fastqs:
        assert fastq.name.startswith(sample_internal_id)
        assert fastq.name.endswith(".fastq.gz")


def test_add_flow_cell_name_to_fastq_file_path(
    tmp_path: Path,
    novaseq_6000_post_1_5_kits_fastq_file_names: list[str],
    novaseq_6000_post_1_5_kits_flow_cell_id: str,
):
    # GIVEN a list of fastq files in a tmp_path that need renaming
    result_file_paths: list[Path] = []
    for fastq_file_name in novaseq_6000_post_1_5_kits_fastq_file_names:
        fastq_file_path = Path(tmp_path, fastq_file_name)
        fastq_file_path.touch()

        # WHEN adding the flow cell name to the fastq file path
        result_file_paths.append(
            _add_flow_cell_name_to_fastq_file_path(
                fastq_file_path=fastq_file_path,
                flow_cell_id=novaseq_6000_post_1_5_kits_flow_cell_id,
            )
        )

    # THEN all the fastq files should have the flow cell id in their names
    assert (
        file.name.startswith(novaseq_6000_post_1_5_kits_flow_cell_id) for file in result_file_paths
    )


@pytest.mark.parametrize(
    "total_1, percentage_1, total_2, percentage_2, expected_result",
    [
        (50, 0.9, 50, 0.7, 0.8),
        (0, 0.0, 0, 0.0, 0.0),
    ],
    ids=["equal_totals", "zero_counts"],
)
def test_get_weighted_average(
    total_1: int, percentage_1: float, total_2: int, percentage_2: float, expected_result: float
):
    # GIVEN total counts and percentages for two metrics

    # WHEN Calculating the weighted average
    result: float = _get_weighted_average(
        total_1=total_1, percentage_1=percentage_1, total_2=total_2, percentage_2=percentage_2
    )

    # THEN The weighted average should be 0.8
    assert math.isclose(result, expected_result, rel_tol=1e-9)


def test_combine_metrics(
    mapped_metric: IlluminaSampleSequencingMetricsDTO,
    undetermined_metric: IlluminaSampleSequencingMetricsDTO,
):
    # GIVEN two metrics

    # WHEN Combining the metrics
    existing_metric: IlluminaSampleSequencingMetricsDTO = _combine_metrics(
        existing_metric=mapped_metric, new_metric=undetermined_metric
    )

    # THEN The existing metric should be updated
    assert existing_metric.total_reads_in_lane == 200
    assert existing_metric.yield_ == 200


@pytest.mark.parametrize(
    "mapped_metrics, undetermined_metrics, expected_result",
    [
        ([], [], []),
        (["mapped_metric"], [], ["mapped_metric"]),
        ([], ["undetermined_metric"], ["undetermined_metric"]),
        (["mapped_metric"], ["undetermined_metric"], ["combined_metric"]),
    ],
    ids=["empty", "only_mapped", "only_undetermined", "both_mapped_and_undetermined"],
)
def test_combine_metrics(
    mapped_metrics: list[str | None],
    undetermined_metrics: list[str | None],
    expected_result: list[str | None],
    request: FixtureRequest,
):
    # GIVEN a combination of mapped and undetermined metrics
    mapped_metrics: list[IlluminaSampleSequencingMetricsDTO | None] = [
        request.getfixturevalue(fixture_name) for fixture_name in mapped_metrics
    ]
    undetermined_metrics: list[IlluminaSampleSequencingMetricsDTO | None] = [
        request.getfixturevalue(fixture_name) for fixture_name in undetermined_metrics
    ]

    # WHEN combining the metrics
    combined_metrics: list[IlluminaSampleSequencingMetricsDTO | None] = (
        combine_sample_metrics_with_undetermined(
            sample_metrics=mapped_metrics, undetermined_metrics=undetermined_metrics
        )
    )

    # THEN the combined metrics should be as expected
    expected_metrics: list[IlluminaSampleSequencingMetricsDTO | None] = [
        request.getfixturevalue(fixture_name) for fixture_name in expected_result
    ]
    assert combined_metrics == expected_metrics


def test_combine_metrics_with_both_mapped_and_undetermined_metrics_different_lanes(
    mapped_metric: IlluminaSampleSequencingMetricsDTO,
    undetermined_metric: IlluminaSampleSequencingMetricsDTO,
):
    # GIVEN one mapped and one undetermined metric in different lanes for a sample
    mapped_metrics = [mapped_metric]
    undetermined_metric.flow_cell_lane = 2
    undetermined_metrics = [undetermined_metric]

    # WHEN combining them
    combined_metrics = combine_sample_metrics_with_undetermined(
        sample_metrics=mapped_metrics, undetermined_metrics=undetermined_metrics
    )

    # THEN two metrics should be returned
    assert len(combined_metrics) == 2
    assert combined_metrics[0].flow_cell_lane != combined_metrics[1].flow_cell_lane


def test_get_undetermined_fastqs(tmp_demultiplexed_novaseq_6000_post_1_5_kits_path: Path):
    # GIVEN a demultiplexed run with undetermined fastq files

    # WHEN getting the undetermined fastq files
    undetermined_fastqs: list[Path] = get_undetermined_fastqs(
        lane=1, demultiplexed_run_path=tmp_demultiplexed_novaseq_6000_post_1_5_kits_path
    )

    # THEN the result should be the undetermined fastq file
    assert len(undetermined_fastqs) == 2
    assert all(
        undetermined_fastq.name.startswith("Undetermined")
        for undetermined_fastq in undetermined_fastqs
    )


def test_create_delivery_file_in_flow_cell_directory(tmp_path: Path):
    # GIVEN a temporary directory without a delivery file
    flow_cell_directory: Path = tmp_path
    assert not (flow_cell_directory / DemultiplexingDirsAndFiles.DELIVERY).exists()

    # WHEN creating a delivery file inside the temporary directory
    create_delivery_file_in_flow_cell_directory(flow_cell_directory)

    # THEN a delivery file should exist in the temporary directory
    assert (flow_cell_directory / DemultiplexingDirsAndFiles.DELIVERY).exists()


@pytest.mark.parametrize(
    "sample_fastq_file_name",
    [
        "novaseq_6000_post_1_5_kits_fastq_file_lane_1",
        "novaseq_6000_post_1_5_kits_fastq_file_lane_1_with_flow_cell_id",
    ],
    ids=["no_flow_cell_id", "with_flow_cell_id"],
)
def test_get_lane_from_sample_fastq_file_path(sample_fastq_file_name: str, request: FixtureRequest):
    # GIVEN a sample fastq path and a lane
    fastq_file: str = request.getfixturevalue(sample_fastq_file_name)
    sample_fastq_path = Path(fastq_file)
    expected_lane: int = 1

    # WHEN we get lane from the sample fastq file path
    result_lane: int = get_lane_from_sample_fastq(sample_fastq_path)

    # THEN we should get the correct lane
    assert result_lane == expected_lane


def test_get_q30_threshold():
    # GIVEN a specific sequencer type
    sequencer_type: Sequencers = Sequencers.HISEQGA

    # WHEN getting the Q30 threshold for the sequencer type
    q30_threshold: int = get_q30_threshold(sequencer_type)

    # THEN the correct Q30 threshold should be returned
    assert q30_threshold == FLOWCELL_Q30_THRESHOLD[sequencer_type]



from enum import Enum
from typing import Callable

from cg.constants import Workflow
from cg.services.sequencing_qc_service.quality_checks.utils import (
    any_sample_in_case_has_reads,
    case_pass_sequencing_qc,
    sample_pass_sequencing_qc,
)
from cg.store.models import Case


class QualityCheck(Enum):
    """
    Parent class for the quality checks.
    """


class SequencingQCCheck(QualityCheck):
    CASE_PASSES: Callable = case_pass_sequencing_qc
    SAMPLE_PASSES: Callable = sample_pass_sequencing_qc
    ANY_SAMPLE_IN_CASE_HAS_READS: Callable = any_sample_in_case_has_reads


def get_sequencing_quality_check_for_case(case: Case) -> Callable:
    """Return the appropriate sequencing quality checks for the workflow for a case."""
    workflow: Workflow = case.data_analysis

    case_passes_workflows = [
        Workflow.BALSAMIC,
        Workflow.BALSAMIC_PON,
        Workflow.BALSAMIC_QC,
        Workflow.BALSAMIC_UMI,
        Workflow.MIP_DNA,
        Workflow.MIP_RNA,
        Workflow.RAREDISEASE,
        Workflow.RNAFUSION,
        Workflow.TOMTE,
    ]

    any_sample_in_case_has_reads_workflows = [
        Workflow.FLUFFY,
        Workflow.RAW_DATA,
        Workflow.MICROSALT,
        Workflow.MUTANT,
        Workflow.TAXPROFILER,
    ]

    if workflow in case_passes_workflows:
        return SequencingQCCheck.CASE_PASSES
    elif workflow in any_sample_in_case_has_reads_workflows:
        return SequencingQCCheck.ANY_SAMPLE_IN_CASE_HAS_READS
    raise ValueError(f"Workflow {workflow} does not have a sequencing quality check.")


def get_sample_sequencing_quality_check() -> Callable:
    return SequencingQCCheck.SAMPLE_PASSES


def run_quality_checks(quality_checks: list[QualityCheck], **kwargs) -> bool:
    return all(quality_check(**kwargs) for quality_check in quality_checks)



from sqlalchemy.orm import Query

from cg.constants import Workflow
from cg.store.filters.status_order_filters import filter_orders_by_ticket_id
from cg.store.models import Order
from cg.store.store import Store


def test_filter_orders_by_ticket_no_matching_ticket(base_store: Store, non_existent_id: str):
    """Test that no cases are returned when filtering by a non-existent ticket."""
    # GIVEN a store containing orders with no matching ticket id
    order = Order(
        id=1,
        customer_id=1,
        ticket_id=1,
    )
    base_store.session.add(order)
    base_store.session.commit()
    order_query: Query = base_store._get_query(table=Order)

    # WHEN filtering orders by a non-existent ticket
    filtered_orders: Query = filter_orders_by_ticket_id(orders=order_query, ticket_id=2)

    # THEN the query should return no order
    assert filtered_orders.count() == 0


def test_filter_orders_by_ticket_id_matching_ticket(base_store: Store, ticket_id: str):
    """Test that the order is returned when filtering by an existing ticket id."""

    order = Order(
        id=1,
        customer_id=1,
        ticket_id=int(ticket_id),
    )
    base_store.session.add(order)
    base_store.session.commit()

    # GIVEN a store containing an order with a matching ticket id
    order_query: Query = base_store._get_query(table=Order)

    # WHEN filtering orders by an existing ticket id
    filtered_orders: Query = filter_orders_by_ticket_id(
        orders=order_query, ticket_id=int(ticket_id)
    )

    # THEN the query should return cases with the matching ticket
    assert filtered_orders.count() == 1
    assert filtered_orders.first().ticket_id == int(ticket_id)



from pydantic import BaseModel


class StatinaUploadFiles(BaseModel):
    result_file: str
    multiqc_report: str | None = None
    segmental_calls: str | None = None


class SequencingRunQ30AndReads(BaseModel):
    """Summaries for Q30 and reads of a sequencing run."""

    total_reads_on_flow_cell: int
    average_q30_across_samples: float

    def passes_read_threshold(self, threshold: int) -> bool:
        """Check if the number of reads is above a threshold."""
        return self.total_reads_on_flow_cell >= threshold

    def passes_q30_threshold(self, threshold: float) -> bool:
        """Check if the Q30 is above a threshold."""
        return self.average_q30_across_samples >= threshold



from typing import Generator

import pytest

from cg.constants.constants import ControlOptions
from cg.store.store import Store
from tests.store_helpers import StoreHelpers


@pytest.fixture
def applications_store(store: Store, helpers: StoreHelpers) -> Store:
    """Return a store populated with applications from excel file"""
    app_tags: list[str] = ["PGOTTTR020", "PGOTTTR030", "PGOTTTR040"]
    for app_tag in app_tags:
        helpers.ensure_application(store=store, tag=app_tag)
    return store


@pytest.fixture
def microbial_store(store: Store, helpers: StoreHelpers) -> Generator[Store, None, None]:
    """Populate a store with microbial application tags"""
    microbial_active_apptags = ["MWRNXTR003", "MWGNXTR003", "MWMNXTR003", "MWLNXTR003"]
    microbial_inactive_apptags = ["MWXNXTR003", "VWGNXTR001", "VWLNXTR001"]

    for app_tag in microbial_active_apptags:
        helpers.ensure_application(store=store, tag=app_tag, prep_category="mic", is_archived=False)

    for app_tag in microbial_inactive_apptags:
        helpers.ensure_application(store=store, tag=app_tag, prep_category="mic", is_archived=True)

    return store


@pytest.fixture
def mutant_store(store: Store, helpers: StoreHelpers) -> Store:
    # Add mutant application and application_version
    application = helpers.add_application(
        store=store, application_tag="VWGDPTR001", target_reads=2000000, percent_reads_guaranteed=1
    )

    # Add cases
    case_qc_pass = helpers.add_case(store=store, name="case_qc_pass", internal_id="case_qc_pass")
    case_qc_fail = helpers.add_case(store=store, name="case_qc_fail", internal_id="case_qc_fail")

    case_qc_fail_with_failing_controls = helpers.add_case(
        store=store,
        name="case_qc_fail_with_failing_controls",
        internal_id="case_qc_fail_with_failing_controls",
    )

    # Add samples
    sample_qc_pass = helpers.add_sample(
        store=store,
        internal_id="sample_qc_pass",
        name="sample_qc_pass",
        control=ControlOptions.EMPTY,
        reads=861966,
        application_tag=application.tag,
    )

    sample_qc_fail = helpers.add_sample(
        store=store,
        internal_id="sample_qc_fail",
        name="sample_qc_fail",
        control=ControlOptions.EMPTY,
        reads=438776,
        application_tag=application.tag,
    )

    external_negative_control_qc_pass = helpers.add_sample(
        store=store,
        internal_id="external_negative_control_qc_pass",
        name="external_negative_control_qc_pass",
        control=ControlOptions.NEGATIVE,
        reads=20674,
        application_tag=application.tag,
    )

    internal_negative_control_qc_pass = helpers.add_sample(
        store=store,
        internal_id="internal_negative_control_qc_pass",
        name="internal_negative_control_qc_pass",
        control=ControlOptions.NEGATIVE,
        reads=0,
        application_tag=application.tag,
    )

    sample_qc_pass_with_failing_controls = helpers.add_sample(
        store=store,
        internal_id="sample_qc_pass_with_failing_controls",
        name="sample_qc_pass",
        control=ControlOptions.EMPTY,
        reads=861966,
        application_tag=application.tag,
    )

    internal_negative_control_qc_fail = helpers.add_sample(
        store=store,
        internal_id="internal_negative_control_qc_fail",
        name="internal_negative_control_qc_fail",
        control=ControlOptions.NEGATIVE,
        reads=3000,
        application_tag=application.tag,
    )

    external_negative_control_qc_fail = helpers.add_sample(
        store=store,
        internal_id="external_negative_control_qc_fail",
        name="external_negative_control_qc_fail",
        control=ControlOptions.NEGATIVE,
        reads=200000,
        application_tag=application.tag,
    )

    # Add CaseSample relationships
    # case_qc_pass
    helpers.add_relationship(store=store, case=case_qc_pass, sample=sample_qc_pass)
    helpers.add_relationship(
        store=store, case=case_qc_pass, sample=external_negative_control_qc_pass
    )

    # case_qc_fail
    helpers.add_relationship(store=store, case=case_qc_fail, sample=sample_qc_fail)
    helpers.add_relationship(
        store=store, case=case_qc_fail, sample=external_negative_control_qc_pass
    )

    # case_qc_fail_with_failing_controls
    helpers.add_relationship(
        store=store,
        case=case_qc_fail_with_failing_controls,
        sample=sample_qc_pass_with_failing_controls,
    )
    helpers.add_relationship(
        store=store,
        case=case_qc_fail_with_failing_controls,
        sample=external_negative_control_qc_fail,
    )

    return store



import datetime as dt
from pathlib import Path

import pytest

from cg.apps.housekeeper.hk import HousekeeperAPI
from cg.apps.housekeeper.models import InputBundle
from cg.constants import SequencingFileTag, Workflow
from cg.constants.constants import CaseActions
from cg.meta.workflow.fluffy import FluffyAnalysisAPI
from cg.models.cg_config import CGConfig
from cg.store.models import Case, Sample
from cg.store.store import Store
from tests.store_helpers import StoreHelpers


@pytest.fixture(scope="function")
def fluffy_case_id_existing(selected_novaseq_6000_post_1_5_kits_case_ids):
    return selected_novaseq_6000_post_1_5_kits_case_ids[0]


@pytest.fixture(scope="function")
def fluffy_case_id_non_existing():
    return "nakedmolerat"


@pytest.fixture(scope="function")
def fluffy_sample_lims_id(selected_novaseq_6000_post_1_5_kits_sample_ids):
    return selected_novaseq_6000_post_1_5_kits_sample_ids[0]


@pytest.fixture(scope="function")
def fluffy_success_output_summary(tmpdir_factory):
    output_dir = tmpdir_factory.mktemp("output")
    file_path = Path(output_dir, "summary.csv")
    file_path.touch(exist_ok=True)
    return file_path


@pytest.fixture(scope="function")
def fluffy_success_output_aberrations(tmpdir_factory):
    output_dir = tmpdir_factory.mktemp("output")
    file_path = Path(output_dir, "WCXpredict_aberrations.filt.bed")
    file_path.touch(exist_ok=True)
    return file_path


@pytest.fixture
def bcl_convert_samplesheet_path() -> Path:
    return Path("tests", "fixtures", "data", "bcl_convert_sample_sheet.csv")


@pytest.fixture
def sample() -> Sample:
    return Sample(
        name="sample_name",
        order="sample_project",
        control="positive",
        last_sequenced_at=dt.datetime.now(),
    )


@pytest.fixture(scope="function")
def fluffy_fastq_file_path(config_root_dir):
    path = Path(config_root_dir)
    path.mkdir(parents=True, exist_ok=True)
    fastq_path = Path(path, "fastq.fastq.gz")
    fastq_path.touch(exist_ok=True)
    return fastq_path


@pytest.fixture(scope="function")
def deliverables_yaml_path():
    return Path("tests/fixtures/apps/fluffy/deliverables.yaml")


@pytest.fixture
def fluffy_hermes_deliverables_response_data(
    create_multiqc_html_file,
    fluffy_case_id_existing,
    fluffy_sample_lims_id,
    fluffy_success_output_summary,
    fluffy_success_output_aberrations,
    timestamp_yesterday,
):
    return InputBundle(
        **{
            "files": [
                {
                    "path": fluffy_success_output_summary.as_posix(),
                    "tags": ["metrics", fluffy_case_id_existing, "nipt"],
                },
                {
                    "path": create_multiqc_html_file.as_posix(),
                    "tags": ["multiqc-html", fluffy_case_id_existing, "nipt"],
                },
                {
                    "path": fluffy_success_output_aberrations.as_posix(),
                    "tags": ["wisecondor", "cnv", fluffy_sample_lims_id, "nipt"],
                },
            ],
            "created": timestamp_yesterday,
            "name": fluffy_case_id_existing,
        }
    )


@pytest.fixture(scope="function")
def fluffy_fastq_hk_bundle_data(
    fluffy_fastq_file_path: Path,
    fluffy_sample_lims_id: str,
    novaseq_6000_post_1_5_kits_flow_cell_id: str,
) -> dict:
    return {
        "name": fluffy_sample_lims_id,
        "created": dt.datetime.now(),
        "version": "1.0",
        "files": [
            {
                "path": fluffy_fastq_file_path.as_posix(),
                "tags": [SequencingFileTag.FASTQ, novaseq_6000_post_1_5_kits_flow_cell_id],
                "archive": False,
            }
        ],
    }


@pytest.fixture(scope="function")
def fluffy_samplesheet_bundle_data(
    novaseq_6000_post_1_5_kits_correct_sample_sheet_path: Path,
    novaseq_6000_post_1_5_kits_flow_cell_id: str,
) -> dict:
    return {
        "name": novaseq_6000_post_1_5_kits_flow_cell_id,
        "created": dt.datetime.now(),
        "version": "1.0",
        "files": [
            {
                "path": str(novaseq_6000_post_1_5_kits_correct_sample_sheet_path),
                "tags": [novaseq_6000_post_1_5_kits_flow_cell_id, "samplesheet"],
                "archive": False,
            }
        ],
    }


@pytest.fixture(scope="function")
def fluffy_store(
    store_with_illumina_sequencing_data: Store,
    fluffy_case_id_existing: str,
    fluffy_sample_lims_id: str,
) -> Store:
    case: Case = store_with_illumina_sequencing_data.get_case_by_internal_id(
        internal_id=fluffy_case_id_existing
    )
    case.data_analysis = Workflow.FLUFFY
    case.links[0].sample.last_sequenced_at = dt.datetime.now()
    case.links[0].sample.reads = 50
    case.action = CaseActions.ANALYZE
    return store_with_illumina_sequencing_data


@pytest.fixture(scope="function")
def fluffy_context(
    cg_context: CGConfig,
    helpers: StoreHelpers,
    real_housekeeper_api: HousekeeperAPI,
    fluffy_samplesheet_bundle_data: dict,
    fluffy_fastq_hk_bundle_data: dict,
    fluffy_case_id_existing: str,
    fluffy_sample_lims_id: str,
    fluffy_store: Store,
) -> CGConfig:
    cg_context.housekeeper_api_ = real_housekeeper_api
    cg_context.status_db_ = fluffy_store
    fluffy_analysis_api = FluffyAnalysisAPI(config=cg_context)
    helpers.ensure_hk_version(
        fluffy_analysis_api.housekeeper_api, bundle_data=fluffy_samplesheet_bundle_data
    )
    helpers.ensure_hk_version(fluffy_analysis_api.housekeeper_api, fluffy_fastq_hk_bundle_data)
    cg_context.meta_apis["analysis_api"] = fluffy_analysis_api
    return cg_context



"""Tests for the meta PdcAPI"""

from unittest import mock
import pytest
from cg.constants import EXIT_FAIL
from cg.constants.process import EXIT_WARNING
from cg.exc import (
    PdcError,
)
from cg.models.cg_config import CGConfig
from tests.conftest import create_process_response


def test_validate_is_dsmc_process_running(cg_context: CGConfig):
    """Tests checking if a Dsmc process is running when no Dsmc process is running."""
    # GIVEN an instance of the PDC API
    pdc_service = cg_context.pdc_service

    # GIVEN no Dsmc process is running

    # WHEN checking if Dsmc is running
    is_dmsc_running: bool = pdc_service.validate_is_dsmc_running()

    # THEN return false
    assert not is_dmsc_running


@mock.patch("cg.utils.commands.Process.run_command")
def test_archive_file_to_pdc(mock_process, cg_context: CGConfig, backup_file_path):
    """Tests execution command to archive file to PDC"""
    # GIVEN an instance of the PDC API

    pdc_service = cg_context.pdc_service
    pdc_service.process = mock_process

    # WHEN archiving a file to PDC
    pdc_service.archive_file_to_pdc(backup_file_path)

    # THEN a dsmc process should be started with the parameter 'archive'
    pdc_service.process.run_command.assert_called_once_with(
        parameters=["archive", backup_file_path], dry_run=False
    )


@mock.patch("cg.utils.commands.Process.run_command")
def test_query_pdc(mock_process, cg_context: CGConfig, backup_file_path):
    """Tests execution command to query files to PDC"""
    # GIVEN an instance of the PDC API
    pdc_service = cg_context.pdc_service
    pdc_service.process = mock_process

    # WHEN querying PDC
    pdc_service.query_pdc(backup_file_path)

    # THEN a dsmc process should be started with parameters 'q archive'
    mock_process.run_command.assert_called_once_with(
        parameters=["q", "archive", backup_file_path], dry_run=False
    )


@mock.patch("cg.utils.commands.Process.run_command")
def test_retrieve_file_from_pdc(mock_process, cg_context: CGConfig, backup_file_path):
    """Tests execution command to retrieve files from PDC"""
    # GIVEN an instance of the PDC API
    pdc_service = cg_context.pdc_service
    pdc_service.process = mock_process

    # WHEN retrieving a file form PDC
    pdc_service.retrieve_file_from_pdc(backup_file_path)

    # THEN a dsmc process should be started with parameters 'retrieve -replace=yes'
    mock_process.run_command.assert_called_once_with(
        parameters=["retrieve", "-replace=yes", backup_file_path], dry_run=False
    )


def test_run_dsmc_command_fail(cg_context: CGConfig):
    """Test that non-zero, non-warning exit-codes raise an error."""
    # GIVEN an instance of the PDC API
    pdc_service = cg_context.pdc_service

    # GIVEN an exit code signifying failure
    with pytest.raises(PdcError), mock.patch(
        "cg.utils.commands.subprocess.run",
        return_value=create_process_response(return_code=EXIT_FAIL),
    ):
        # WHEN running a dsmc command
        pdc_service.run_dsmc_command(["archive", "something"])

        # THEN the appropriate error should have been raised


def test_run_dsmc_command_warning(cg_context: CGConfig, caplog):
    """Test that warning exit-codes do not raise an error."""
    # GIVEN an instance of the PDC API
    pdc_service = cg_context.pdc_service

    # GIVEN an exit code signifying a warning
    with mock.patch(
        "cg.utils.commands.subprocess.run",
        return_value=create_process_response(return_code=EXIT_WARNING),
    ):
        # WHEN running a dsmc command
        pdc_service.run_dsmc_command(["archive", "something"])

    # THEN the warning should have been logged
    assert "WARNING" in caplog.text



"""Tests for the compression files module"""

import logging

from cg.meta.compress import files
from cg.models import CompressionData


def test_get_fastq_files(populated_compress_fastq_api, sample):
    """Test get_fastq_files method when there are FASTQ files"""
    compress_api = populated_compress_fastq_api
    # GIVEN a sample_id and a hk api populated with fastq bundle

    # WHEN fetching the fastq files
    version_obj = compress_api.hk_api.get_latest_bundle_version(bundle_name=sample)
    fastq_dict = files.get_fastq_files(sample_id=sample, version_obj=version_obj)
    run = list(fastq_dict.keys())[0]

    # THEN the fastq files will be in the dictionary
    assert set(fastq_dict[run].keys()) == set(["compression_data", "hk_first", "hk_second"])
    # THEN assert that a CompressionData object was returned
    assert isinstance(fastq_dict[run]["compression_data"], CompressionData)


def test_get_fastq_files_no_files(compress_api, sample_hk_bundle_no_files, sample, helpers, caplog):
    """Test get_fastq_files method when no FASTQ files are found"""
    caplog.set_level(logging.DEBUG)
    # GIVEN a sample_id
    sample_id = sample
    hk_api = compress_api.hk_api
    # GIVEN a bundle without files
    assert sample_hk_bundle_no_files["files"] == []
    helpers.ensure_hk_bundle(hk_api, sample_hk_bundle_no_files)

    # WHEN fetching the fastq files
    version_obj = compress_api.hk_api.get_latest_bundle_version(bundle_name=sample)
    fastq_dict = files.get_fastq_files(sample_id=sample_id, version_obj=version_obj)

    # THEN assert that None is returned since there where not two files
    assert fastq_dict is None
    # THEN assert that the correct information is returned
    assert f"Could not find FASTQ files for {sample}" in caplog.text



"""Delivery API path fixtures."""

from pathlib import Path

import pytest

from cg.constants import FileExtensions


@pytest.fixture
def delivery_fastq_file(tmp_path: Path, sample_id: str) -> Path:
    file = Path(tmp_path, f"FC_{sample_id}_L001_R1_001{FileExtensions.FASTQ_GZ}")
    file.touch()
    return file


@pytest.fixture
def delivery_case_fastq_file(tmp_path: Path, sample_id: str) -> Path:
    """
    This represents a fastq file stored on a case bundle. Mutant stored file like this in the past.
    This fixture servers the purpose to make sure these files are not fetched during delivery.
    """
    file = Path(tmp_path, f"{sample_id}_concat_{FileExtensions.FASTQ_GZ}")
    file.touch()
    return file


@pytest.fixture
def delivery_bam_file(tmp_path: Path, sample_id: str) -> Path:
    file = Path(tmp_path, f"{sample_id}_R1_001{FileExtensions.BAM}")
    file.touch()
    return file


@pytest.fixture
def delivery_another_fastq_file(tmp_path: Path, another_sample_id: str) -> Path:
    file = Path(tmp_path, f"FC_{another_sample_id}L001_R1_001{FileExtensions.FASTQ_GZ}")
    file.touch()
    return file


@pytest.fixture
def delivery_another_case_fastq_file(tmp_path: Path, another_sample_id: str) -> Path:
    """
    This represents a fastq file stored on a case bundle. Mutant stored file like this in the past.
    This fixture servers the purpose to make sure these files are not fetched during delivery.
    """
    file = Path(tmp_path, f"{another_sample_id}_concat_{FileExtensions.FASTQ_GZ}")
    file.touch()
    return file


@pytest.fixture
def delivery_another_bam_file(tmp_path: Path, another_sample_id: str) -> Path:
    file = Path(tmp_path, f"{another_sample_id}_R1_001{FileExtensions.BAM}")
    file.touch()
    return file


@pytest.fixture
def delivery_spring_file(tmp_path: Path, sample_id: str) -> Path:
    file = Path(tmp_path, f"{sample_id}_S1_001{FileExtensions.SPRING}")
    file.touch()
    return file


@pytest.fixture
def delivery_another_spring_file(tmp_path: Path, another_sample_id: str) -> Path:
    file = Path(tmp_path, f"{another_sample_id}_S1_001{FileExtensions.SPRING}")
    file.touch()
    return file


@pytest.fixture
def delivery_report_file(tmp_path: Path, case_id: str) -> Path:
    file = Path(tmp_path, f"{case_id}_delivery-report{FileExtensions.HTML}")
    file.touch()
    return file


@pytest.fixture
def delivery_cram_file(tmp_path: Path, sample_id: str) -> Path:
    file = Path(tmp_path, f"{sample_id}{FileExtensions.CRAM}")
    file.touch()
    return file


@pytest.fixture
def delivery_another_cram_file(tmp_path: Path, another_sample_id: str) -> Path:
    file = Path(tmp_path, f"{another_sample_id}{FileExtensions.CRAM}")
    file.touch()
    return file


@pytest.fixture
def delivery_ticket_dir_path(tmp_path: Path, ticket_id: str) -> Path:
    return Path(tmp_path, ticket_id)


@pytest.fixture
def delivery_consensus_sample_file(tmp_path: Path, sample_id: str) -> Path:
    file = Path(tmp_path, f"{sample_id}_consensus_sample{FileExtensions.VCF}")
    file.touch()
    return file


@pytest.fixture
def delivery_another_consensus_sample_file(tmp_path: Path, another_sample_id: str) -> Path:
    file = Path(tmp_path, f"{another_sample_id}_consensus_sample{FileExtensions.VCF}")
    file.touch()
    return file


@pytest.fixture
def delivery_vcf_report_file(tmp_path: Path, sample_id: str) -> Path:
    file = Path(tmp_path, f"{sample_id}_vcf_report{FileExtensions.VCF}")
    file.touch()
    return file


@pytest.fixture
def delivery_another_vcf_report_file(tmp_path: Path, another_sample_id: str) -> Path:
    file = Path(tmp_path, f"{another_sample_id}_vcf_report{FileExtensions.VCF}")
    file.touch()
    return file
